#!/usr/bin/env perl

# $Id: eBayAgent-de.pl,v 1.1.2.1 2004/11/26 14:23:44 sturnus Exp $

=head1 NAME

eBayAgent.pl - last bid automation for eBay users

=head1 SYNOPSIS

B<eBayAgent.pl [-D] [-v] [-q] [-k] -i item [-u user -p pass] [-l country] [-m [-M email]] [-S [smtpserver]] [-P [proxy]] [-f useragent] [-r rcfile] [-c] [-t] [-b bid [-s [0]] [-g gap] [-d time] [-a time] [-N] [-h retry -H time] [-n] [-w time | -W time] [-B] [-E] [-A]] [-T time -z] [-R]>

Item selection option:
  -i, --item         eBay item number

User configuration options:
  -u, --user         eBay username
  -p, --pass         eBay password
  -l, --locale       eBay's locale web site to use (default "com", type "list" to see which ones are available)
  -m, --mail         mail the log messages to the user specified with -u
  -M, --Mailuser     specify the e-mail address to use (override the eBay username)
  -S, --Sendmail     alternate e-mail sending method via stmpserver if given (localhost by default)
  -P, --Proxy        specify the proxy information
  -f, --fake_useragent  allow the user to specify the useragent used
  -r, --rcfile       specify rcfile to read default values into, default is .ebayrc

Bid options:
  -b, --bid          maximum bid amount (in the currency of the bid)
  -s, --smart        if no one did bid on the item yet, will place a first bid at lowest amount before entering the last bid mode
  -g, --gap          time gap (in seconds) between bid placement and end of auction (default value: 15)
  -d, --dcheck       check "dcheck" seconds before end of gap for validity of our bid (default value: 15)
  -a, --acheck       check "acheck" seconds after end of bid to see who won (default value: 15)
  -N, --Now          place a 'Buy It Now' bid (bid amount must be equal to 'Buy It Now' value)
  -h, --httpretry    number of HTTP retries to perform on an URL (default: 3)
  -H, --Hrdelay      time to wait to try another HTTP retry (default: 1s)
  -n, --ntponly      do not trust eBay's time after first request (require that your system use some form of time synchronisation such as NTP)
  -w, --watch        watch item price changes every "watch" minutes to insure current price is not over max bidding price
  -W, --Watch        same as the --watch option, but notify of every price change
  -T, --Track        keep track of price changes for the item  (does not authorize to place a bid); check the eBay web site every "Track" minutes (default: 30)
  -z, --zealous      when in Track or w/Watch mode notify even of all changes in status
  -B, --Bidnow       perform a bid at time of program call on specified item for specified amount
  -E, --Exitstatus   change the exit status in case you won the bid (useful for shell scripting)
  -A, --Adult        allow bid for adult items

Misc options:
  -c, --check        print the item information and exit (default when no other mode specified)
  -t, --time         print eBay time and exit
  -D, --Disclaimer   print program disclaimer and exit
  -v, --version      print program version number and exit
  -q, --quieter      perform operations without displaying time remaining information
  -k, --keypress     when exiting, wait on a key press before exiting
  -R, --Rest         perform the bid process in the background (useful when no terminal is available)

=cut

# This file is Copyright (C) 2001-2004 Andrew Wansink & Martial Michel
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

############################################################
## Required Packages

#LWP;
unless ( eval "use LWP; 1" )
{
        error_print("\"libwww-perl\" is not available on your Perl installation\nPlease see \"http://search.cpan.org/search?mode=module&query=lwp\" for installation information");
        error_exit();
}
use HTTP::Cookies; # Part of LWP

#Date::Parse;
unless ( eval "use Date::Parse; 1" )
{
        error_print("\"TimeDate\" is not available on your Perl installation\nPlease see \"http://search.cpan.org/search?mode=module&query=date%3A%3Aparse\" for installation information");
        error_exit();
}

#Getopt::Long;
unless ( eval "use Getopt::Long; 1" )
{
        error_print("\"Getopt-Long\" is not available on your Perl installation\nPlease see \"http://search.cpan.org/search?mode=module&query=getopt%3A%3Along\" for installation information");
        error_exit();
}

#URI::Escape;
unless ( eval "use URI::Escape; 1" )
{
        error_print("\"URI\" is not available on your Perl installation\nPlease see \"http://search.cpan.org/search?mode=module&query=uri\" for installation information");
        error_exit();
}

# Notes:
# - the choice of using ignorecase for Getopt is to make sure that we do
#    not have any problem with m/M or b/B options.
# - in the same spirit, autoabbrev was used so that options are automatically
#   reduced to their first letter (as long as there is no option starting with
#   the same first letter (case sensitive))
$Getopt::Long::autoabbrev = 1;
$Getopt::Long::ignorecase = 0;

use strict;

############################################################
## Variables

my $version = "0.9.10b";
my $rpm_release = "1";

# Simply allow you to bid under the minimum bid amount
# If 'debug' value:
# >= 1:  is more verbose/not silent
# >= 2:  store web page to disk
# >= 3:  print ALL cookies & FORM contents (and keep the Cookie Jar file)
my $debug = 0;
# If 'donotbid' and 'debug' are set: will not perform the bidding process
my $debug_donotbid = 0;
my $debugfilecounter = 0;
my $cjar_file = "/tmp/eBayAgent-CookieJar-$$";

my ($useragent, $cjar);

my $gap = 15;
my $dcheck = 15;
my $acheck = 15;
my $httpretry = 3;
my $Hrdelay = 1;
my $httpretry_inc = 0;
my $httpretry_msg = "";
my $httpretry_hmsg = "Bad HTTP content";
my $ntponly = 0;
my $ntponly_endtime = 0;
my $watch = 0;
my $Watch = 0;
my $watch_msg = "W/watch mode";
my $watchprice = 0;
my $watchtimelap = 0;
my $watchchecktime = 0;
my $watchbadhttp = 0;
my $Track = -1;
my $Exitstatus = 0;
my $Exitstatus_checktext = "Auction has ended";
my $Exitstatus_won  = 99;
my $Exitstatus_lost = 88;
my $Adult = 0;
my $locale = "com";
my $zealous = 0;
my $zealous_on = 0;
my $zealous_text = "";
my $Rest = 0;

my $item     = "";
my $bid	     = "";
my $user     = "";
my $pass     = "";
my $check    = 0;
my $timetest = 0;
my $mail     = 0;
my $Mailuser = "";
my $Sendmail = -1;
my $Proxy    = "-1";
my $fake_useragent = "";
my $Now      = 0;
my $Bidnow   = 0;
my $smart    = -1;
my $rcfileb  = ".ebayrc";
my $rcfile   = "./.ebayrc";
my $biddingin_text = "Bidding in";
my $Watchpricechange_text = "Watch price change";
my $iseBayMotors = 0;
my $isnewLayout = 0;

my $currency = "";
my $silent = 0;
my $item_check_baseurl =
        "http://cgi.ebay.com/ws/eBayISAPI.dll?ViewItem&item=";
#        "http://cgi.ebay.com/aw-cgi/eBayISAPI.dll?ViewItem&item=";
my $item_bid_baseurl = 
        'http://offer.ebay.com/ws/eBayISAPI.dll';
# Was:  'http://cgi.ebay.com/aw-cgi/eBayISAPI.dll'
my $item_bid_baseurl2 =
	'http://cgi.ebay.com/aw-cgi/eBayISAPI.dll';
my $item_bid_baseurl3 = 
	'http://signin.ebay.com/ws1/eBayISAPI.dll';
# Was:  'http://signin.ebay.com/aw-cgi/eBayISAPI.dll';
my $item_bidurl = "";
my $item_bidurl2 = "";
# A new url for live extraction of the bid page

my $eBM_item_bid_baseurl = 
        'http://cgi.ebay.com/ebaymotors/aw-cgi/eBayISAPI.dll';
my $max_clockskew = 7;
my $watchbadclockreturn = -999999;
my $skipsubdata = 0;

my $clearline = "                                                          \r";
my $flush_state;
my $lineskip = "";
my $cutchar = '|';

my $quieter = 0;
my $keypress = 0;

# regular expression for replacing '\s' 
# includes non breaking space in M$ code page 1252
my $s_re = qr/[\s\xA0]/;

############################################################
## Main


title();

# Just a quick method to see which letters are still available :)
# ABCDEFGHIJKLMNOPQRSTUVWXYZ abcdefghijklmnopqrstuvwxyz
# AB DE  H    MN P RST  WX   abcd fghi klmn pqrstuvw  z
GetOptions
	(
         'Adult'       => sub { $Adult = 1; },
	 'Bidnow'      => sub { $Bidnow = 1; },
	 'Disclaimer'  => sub { disclaimer(); },
	 'Exitstatus'  => sub { $Exitstatus = 1; },
	 'Hrdelay=i'   => \$Hrdelay,
	 'Mailuser=s'  => \$Mailuser,
	 'Now'         => sub { $Now = 1; },
         'Proxy:s'     => \$Proxy,
	 'Rest'        => sub { $Rest = 1; },
         'Sendmail:s'  => \$Sendmail,
	 'Track:i'     => \$Track,
         'Watch=i'     => \$Watch,
	 'acheck=i'    => \$acheck,
	 'bid=s'       => \$bid,
	 'check'       => sub { $check = 1; },
	 'dcheck=i'    => \$dcheck,
	 'fake_useragent=s' => \$fake_useragent,
	 'gap=i'       => \$gap,
	 'httpretry=i' => \$httpretry,
	 'item=i'      => \$item,
	 'keypress'    => sub { $keypress = 1; },
	 'locale=s'    => \$locale,
	 'mail'        => sub { $mail = 1; },
	 'ntponly'     => sub { $ntponly = 1;},
	 'pass=s'      => \$pass,
	 'quieter'     => sub { $quieter = 1;},
	 'rcfile=s'    => \$rcfile,
         'smart:i'     => \$smart,
	 'time'        => sub { $timetest = 1; },
	 'user=s'      => \$user,
	 'version'     => sub { exit (0); },
         'watch=i'     => \$watch,
	 'zealous'     => sub { $zealous = 1;},
         # I would rather make this one just debug
         # but that would change --dchecks short -d to -dc
         # so.. use an X
         'Xdebug+'      => \$debug, 
	) || usage();

# Item numbers do not need the "-i" in front of them anymore
if (scalar @ARGV > 0)
{
        $item = shift @ARGV;
        if ($item !~ /^\d+$/)
        {
                error_print("Not an item number ($item)");
                usage();
        }
}
if (scalar @ARGV > 0)
{
        error_print("Only one item can be specified at a time");
        usage();
}

# Default mode if not bidding or Tracking is "check"
if (($bid eq "") && ($Track == -1))
{
        $check = 1;
}

# Track check
if ($Track != -1)
{
	if ($watch)
	{
		error_print("\"Track\" does not allow to be combined with \"watch\"");
		usage();
	}
	if ($Watch)
	{
		error_print("\"Track\" does not allow to be combined with \"Watch\"");
		usage();
	}
	if ($bid ne "")
	{
		error_print("\"Track\" does not allow to place a bid, please see the \"Watch\" option");
		usage();
	}
	if ($Bidnow)
	{
		error_print("\"Track\" does not allow to place a bid");
		usage();
	}
}

# watch OR Watch, not both
if ($watch && $Watch)
{
        error_print("W/watch options are mutualy exclusive");
        usage();
}

# Bidnow and Now are mutualy exclusives
if ($Bidnow && $Now)
{
        error_print("Bidnow and Now options are mutualy exclusive");
        usage();
}

# 'debug' validation
if ( $debug > 0 )
{
        $debug_donotbid = $debug_donotbid;
        $lineskip = "\n";
}
else 
{
        $debug_donotbid = 0;
        $lineskip = "";
}

$rcfile =~ s{^~([^/]*)}{$1?(getpwnam($1))[7]:($ENV{HOME} || $ENV{LOGDIR})}ex;
$rcfile = "$ENV{HOME}/$rcfileb" if (! -e $rcfile);

# read settings from rcfile
if (open (RCFILE, "$rcfile"))
{
	my @lines = <RCFILE>;
        my $watchused = 0;
	
	chomp @lines;
	foreach my $line (@lines)
	{
		next if ($line =~ /^\#/);
		next if ($line =~ /^\s*$/);

		my $key = $1 if ($line =~ s/^\s*(\w+)//);
		# 'X_' options are for XeBayAgent only, ignoring
		next if ($key =~ /^X\_/);

                my $value = "";
                $value = $1 if ($line =~ s/^\s*(.+?)\s*$//);
                if ($line ne "")
                {
                        error_exit("Strange config line for key '$key', please check content (after parsing '$line' remains)\n");
                }

                if ($value eq "")
                {
                        error_exit("'$key' has no value. Options must have a value, including on/off options (1=on/0=off).\n");
                }
		
		if ($key =~ m/pass/)
		{
                        if ($pass ne "")
                        {
                                warn_print("[rcfile] command line override of \"--pass\"");
                        }
                        else
                        {
                                $pass = $value;
                        }
		}
		elsif ($key =~ m/[M|m]ailuser/)
		{ # error in variable name => keep support for old option
                        if ($Mailuser ne "")
                        {
                                warn_print("[rcfile] command line override of \"--Mailuser\"");
                        }
                        else
                        {
                                $Mailuser = $value;
                        }
		}
                elsif ($key =~ m/fake_useragent/)
                { # another *user* option -> to be checked before the real one
                        if ($fake_useragent ne "")
                        {
                                warn_print("[rcfile] command line override of \"--fake_useragent\"");
                        }
                        else
                        {
                                $fake_useragent = $value;
                        }
                }
		elsif ($key =~ m/user/)
		{
                        if ($user ne "")
                        {
                                warn_print("[rcfile] command line override of \"--user\"");
                        }
                        else
                        {
                                $user = $value;
                        }
		}
		elsif ($key =~ m/Adult/)
		{
                        if ($Adult != 0)
                        {
                                warn_print("[rcfile] command line override of \"--Adult\"");
                        }
                        else
                        {
                                $Adult = $value;
                        }
		}
                elsif ($key =~ m/Sendmail/)
                {
                        if ($Sendmail != -1)
                        {
                                warn_print("[rcfile] command line override of \"--Sendmail\"");
                        }
                        else
                        {
                                $Sendmail = $value;
                        }
                }
		elsif ($key =~ m/mail/)
		{
                        if ($mail != 0)
                        {
                                warn_print("[rcfile] command line override of \"--mail\"");
                        }
                        else
                        {
                                $mail = $value;
                        }
		}
		elsif ($key =~ m/gap/)
		{
                        if ($gap != 15)
                        {
                                warn_print("[rcfile] command line override of \"--gap\"");
                        }
                        else
                        {
                                $gap = $value;
                        }
		}
                elsif ($key =~ m/dcheck/)
                {
                        if ($dcheck != 15)
                        {
                                warn_print("[rcfile] command line override of \"--dcheck\"");
                        }
                        else
                        {
                                $dcheck = $value;
                        }
		}
		elsif ($key =~ m/acheck/)
		{
                        if ($acheck != 15)
                        {
                                warn_print("[rcfile] command line override of \"--acheck\"");
                        }
                        else
                        {
                                $acheck = $value;
                        }
		}
		elsif ($key =~ m/httpretry/)
		{
                        if ($httpretry != 3)
                        {
                                warn_print("[rcfile] command line override of \"--httpretry\"");
                        }
                        else
                        {
                                $httpretry = $value;
                        }
		}
		elsif ($key =~ m/Hrdelay/)
		{
                        if ($Hrdelay != 1)
                        {
                                warn_print("[rcfile] command line override of \"--Hrdelay\"");
                        }
                        else
                        {
                                $Hrdelay = $value;
                        }
		}
                elsif ($key =~ m/watch/)
                {
                        $watchused++;
                        if ($watch != 0)
                        {
                                warn_print("[rcfile] command line override of \"--watch\"");
                        }
                        elsif ($Watch != 0)
                        {
                                warn_print("[rcfile] command line override of \"--watch\" option by \"--Watch\"");
                        }
                        else
                        {
                                $watch = $value;
                        }
                }
                elsif ($key =~ m/Watch/)
                {
                        $watchused++;
                        if ($Watch != 0)
                        {
                                warn_print("[rcfile] command line override of \"--Watch\"");
                        }
                        elsif ($watch != 0)
                        {
                                warn_print("[rcfile] command line override of \"--Watch\" option by \"--watch\"");
                        }
                        else
                        {
                                $Watch = $value;
                        }
                }
                elsif ($key =~ m/smart/)
                {
                        if ($smart != -1)
                        {
                                warn_print("[rcfile] command line override of \"--smart\"");
                        }
                        else
                        {
                                $smart = $value;
                        }
                }
                elsif ($key =~ m/Proxy/)
                {
                        if ($Proxy ne "-1")
                        {
                                warn_print("[rcfile] command line override of \"--Proxy\"");
                        }
                        else
                        {
                                $Proxy = $value;
                        }
                }
		elsif ($key =~ m/ntponly/)
		{
                        if ($ntponly != 0)
                        {
                                warn_print("[rcfile] command line override of \"--ntponly\"");
                        }
                        else
                        {
                                $ntponly = $value;
                        }
		}
		elsif ($key =~ m/locale/)
		{
                        if ($locale ne "com")
                        {
                                warn_print("[rcfile] command line override of \"--locale\"");
                        }
                        else
                        {
                                $locale = $value;
                        }
		}
		elsif ($key =~ m/quieter/)
		{
                        if ($quieter != 0)
                        {
                                warn_print("[rcfile] command line override of \"--quieter\"");
                        }
                        else
                        {
                                $quieter = $value;
                        }
		}
		elsif ($key =~ m/keypress/)
		{
                        if ($keypress != 0)
                        {
                                warn_print("[rcfile] command line override of \"--keypress\"");
                        }
                        else
                        {
                                $keypress = $value;
                        }
		}
                else
                {
                        log_message ("Ignoring rcfile key: $key\tval: $value", "Configuration file warning");
		}
        }
        if ($watchused > 1)
        {
                error_print("W/watch options are mutualy exclusive");
                usage();
        }
}
else
{
	log_message ("Couldn't open rcfile: $rcfile: $!", "Missing configuration file");
}

# If no command line or rcfile override of default, 'smart' is off
if ($smart == -1)
{
	$smart = 0;
}

# Mail::Sendmail check
if ($Sendmail != -1)
{
        unless ( eval "use Mail::Sendmail; 1" )
        {
                error_print("Package Mail::Sendmail is not available!!\nPlease see \"http://search.cpan.org/search?dist=Mail-Sendmail\" for installation information");
		error_exit();
        }

        if ($Mailuser !~ /@/)
        {
                error_print("\"Mailuser\" must be a fully qualified e-mail address");
                error_exit();
        }

        if (! (($Sendmail eq "") || ($Sendmail == 1)))
        {
                unshift @{$Mail::Sendmail::mailcfg{'smtp'}} , $Sendmail;
        }
}

# Set max clock skew
# Note: This only is useful for "W/watch"ed items
# MM: Why add a "max clock skew" feature ?
#  With "new_endtime" the number of seconds left according to eBay
#   before the bid ends (asked every W/watch interval)
#   and "end_time" the thought --up to now and the system clock-- seconds
#   left before the bid end
# If new_endtime < end_time
# Then the system have lost some time and therefore may bid after the end
#   of the bid, which is bad => resync
# If new_endtime > end_time
# Then the system have gained some time and therefore may bid too early
#   before the end of the bid ... bad too => resync too
# Now we do not want to resync if the system and eBay have just one second
#   difference, therefore we have a "maximum authorized clock skew value"
#   under which we do not "think" it is required to resync (in both case).
#   It seems wise to give it an arbitrary maximal value (here $max_clockskew's
#   default value) but if we consider the we need to be able to bid $gap
#   seconds before the end of the bid, it is wise to ensure that we reduce
#   that value according to what $gap may be ... so:
$max_clockskew = int ($gap/2) < $max_clockskew ? int ($gap/2) : $max_clockskew;

# Set/Check httpretry variables
if ($httpretry <= 0)
{
	error_print("Invalid value for \"httpretry\" ($httpretry), must be > 0");
	error_exit();
}
if ($Hrdelay <= 0)
{
	error_print("Invalid value for \"Hrdelay\" ($Hrdelay), must be > 0");
	error_exit();
}
$httpretry_msg = "eBay did not return a valid web page content despite $httpretry retries at $Hrdelay seconds interval";

# Set a few defaults when tracking
if ($Track != -1)
{
	$watch = 0;
	$Watch = ($Track == 0) ? 30 : $Track;
	$bid = 0; # safegueard
	$biddingin_text = "Time Remaining";
	$Watchpricechange_text = "Track price change";
	$watch_msg = "Track mode";
}

# W/watch option check
if ($watch)
{
        $watchtimelap = $watch;
}
elsif ($Watch)
{
        $watchtimelap = $Watch;
}
# Note: we know W/watch or Track mode is active if watchtimelap is set

# zealous check
if ($zealous && ($Track == -1) && ($watch == 0) && ($Watch == 0))
{
        warn_print("\"zealous\" option only useful when Track or w/Watch mode selected, ignoring\n");
        $zealous = 0;
}

# Exitstatus check
if ($Exitstatus && ($user eq ""))
{
	error_print("Can not use \"Exitstatus\" option if no \"user\" is provided");
	error_exit();
}

if ($Exitstatus && $keypress)
{
	$keypress = 0;
	error_print("Can not use \"Exitstatus\" and \"keypress\" at the same time");
	error_exit();
}

########################################
# Locale (Part 1)
# Note that we still use the ".com" website to obtain the information
# and then use the locale website for the bid process only

my @localelist = ("com", "de");
if ($locale eq "list")
{
	print &list_locale();
	ok_exit();
}
elsif ($locale eq "com")
{
	# This is the default for now ... nothing to do here (for now)
}
elsif ($locale eq "de")
{
	# Germany
	$item_bidurl =
		'http://offer.ebay.de/ws/eBayISAPI.dll';
	$item_bid_baseurl2 =
		'https://signin.ebay.de/ws/eBayISAPI.dll?co_partnerid=2&amp;siteid=77&amp;UsingSSL=1';
#		'http://signin.ebay.de/aw-cgi/eBayISAPI.dll';
	$item_bid_baseurl3 =
		'http://offer.ebay.de/ws/eBayISAPI.dll';
#		'https://signin.ebay.de/ws/eBayISAPI.dll';
	$eBM_item_bid_baseurl =
		'http://cgi.ebay.de/ebaymotors/aw-cgi/eBayISAPI.dll';
}
else
{
	error_exit("\nError: Invalid \"locale\" value \"$locale\"\n" 
		   . &list_locale());
}
print "(Locale for this bid process is \"$locale\")\n";

########################################
# Check that each of the mandatory options has been assigned a value
if ( (! $timetest) && (! ($check && $item ne "") && ($item eq "" || $bid eq "" || $user eq "" || $pass eq "") && ($Track == -1)))
{
	usage();
}

# Check that the bid amount is valid -- Bids can be in the form of
# integer dollar values (35) or dollars and cents (35.01) or just cents
# (.99)
if ($bid !~ /^(\d+)?(\.\d\d)?$/)
{
	log_message ("Invalid bid amount ($bid)", "Invalid bid amount ($bid)");
	error_exit();
}

########################################
# Optional modules

# POSIX
if ($Rest == 1)
{
        unless ( eval "use POSIX; 1" )
        {
                error_print("\"POSIX\" is not available on your Perl installation\nPlease see \"http://search.cpan.org/search?mode=module&query=posix\" for installation information");
                error_exit();
        }

        deamonize();
}

########################################
##############################
## Starting here we can use the internet
set_useragent();

if ($timetest) {
	timetest();
}
# Note: It is now important to do an "item_check" before placing the bid
#       as this will get the proper cookies in case we have to do an
#       "Adult Login"
my $res = item_check($item);
checkbadhttp_fatal(". Initial item check failed, Exiting");

my $item_name = item_name ($res);
my $end_time  = end_time ($res);

my %add_item_info;
my %item_info = extractii($res->content);

if ($check || ($Track != -1))
{
	log_message(checkitem(%item_info), "Item check");
	bidend_exit(%item_info) if ($check);
}

bidvaliditycheck(%item_info);

if ($Bidnow)
{
        printf("Bidding $currency %.2f for item $item ($item_name)\n", $bid);
        log_message(bid($item, $bid, $user, $pass), sprintf("Bidding $currency %.2f", $bid));
        $res = item_check($item);
	checkbadhttp_fatal(". Can not get information on item after bidding, exiting");
        %item_info = extractii($res->content);
	log_message(checkitem(%item_info), "Item check");
	ok_exit(%item_info);
}

if ($Now)
{
        if ($add_item_info{'buyitnow'} != $bid)
        {
                error_print(sprintf("Bid amount must be \'Buy It Now\' value (of $currency %.2f)", $add_item_info{'buyitnow'}));
                error_exit();
        }

        log_message(buyitnow($item, $bid, $user, $pass), sprintf("Buy It Now $currency %.2f", $bid));
        $res = item_check($item);
        checkbadhttp_fatal(". Can not get information on item after bidding, exiting");
        %item_info = extractii($res->content);
        log_message(checkitem(%item_info), "Item check");
        ok_exit(%item_info);
}

if ($smart && ($Track == -1))
{
        silenton();
        if ($item_info{'current'} == $item_info{'minbid'})
        {
                log_message (sprintf("Placing initial bid for $currency %.2f", $item_info{'minbid'}), sprintf("Smart Init bid at $currency %.2f", $item_info{'minbid'}));
                &bid ($item, $item_info{'minbid'}, $user, $pass);
                $res = item_check($item);
		if (checkbadhttp())
		{
			log_message("Warning: " . $httpretry_msg . ". Can not determine current price. Will continue as if all went ok\n", $httpretry_hmsg);
		}
		else
		{
			%item_info = extractii($res->content);
		}
        }
        silentoff();
}

if ($watchtimelap)
{ # W/watch is active
        $watchprice = $item_info{'current'};
        $watchchecktime = watchchecktime_next();
}
else
{ # W/watch is not active, yet we still want it to check 5min before the end
	$watchchecktime = $end_time - $gap - $dcheck - 300;
	$watch_msg = "5min time synch mode";
}

Track_log_message (sprintf("Bid accepted: Maximum bid of $currency %.2f (Currently: $currency %.2f / Minimum bid: $currency %.2f) for item number $item ($item_name)", $bid, $item_info{'current'}, $item_info{'minbid'}), sprintf("Bid accepted ($currency %.2f)", $bid));

if ($zealous)
{
	# Initial info from which all derives
	$zealous_text = get_zealous_text(%item_info);
}

silenton();
$flush_state = $|;
if ($end_time > (time() + $gap))
{
        # Save the current "flush" state and set to autoflush
        $| = 1;

        # count down until you only have $gap seconds left
        my $docheck = 1;
	my $atomictime = time();
        while ($end_time > ($atomictime + $gap))
        {
                if (($docheck) && ($end_time <= ($atomictime + $gap + $dcheck)))
                {
                        $docheck = 0;
                        print "\n" if (! $quieter);
                        my $res = item_check($item);
			if (checkbadhttp())
			{
				log_message("Warning: " . $httpretry_msg .
					    ". Can not determine price just before bidding. Will still continue bid process\n", $httpretry_msg);
			}
			else
			{
				my %item_info = extractii($res->content);
				bidvaliditycheck(%item_info);
			}
                }

                if (($docheck) && ($atomictime >= $watchchecktime))
                { # W/watch mode (or the 5min/1min check)
                        $watchbadhttp = 1;
                        my $res = item_check($item);
			if (checkbadhttp())
			{
				log_message("Warning: " . $httpretry_msg . ". Since this occured in " . $watch_msg .", the program will continue as if nothing happened\n", $httpretry_hmsg);
			}
                        elsif ($watchbadhttp == 1)
                        {
				my $tmp_endtime = end_time($res);
                                $tmp_endtime =
                                        ($tmp_endtime == $watchbadclockreturn)
                                                ? $end_time : $tmp_endtime;
				my $clock_skew = abs($tmp_endtime - $end_time);
				if ($clock_skew > $max_clockskew)
				{
					log_message("Warning: A clock skew of $clock_skew seconds between eBay and your system has been detected. This value being bigger than the maximum authorized clock skew ($max_clockskew seconds), eBayAgent will correct its ending time to avoid bidding problems.\n\nYou may want to run a \"Time Synchronization Server\" on your system to try to avoid such problems.\nMore information is available at: http://www.ntp.org/\n\nIf you are already running 'ntp' (or another time synchronization mechanism) and believe to have received this message in error, please run the script using the \'ntponly\' option to avoid seeing such messages\n", "Warning: $clock_skew seconds clock skew");
					$end_time = $tmp_endtime;
				}

				if ($watchtimelap)
				{ # W/watch is active
					my %item_info = extractii($res->content);
					
					if ($item_info{'current'} > $watchprice)
					{
						if ($Watch)
						{
							print "\n" if (! $quieter);
							log_message(checkitem(%item_info) . (($Track == -1) ? sprintf("\n\nReminder: Your maximum bid for this item is $currency %.2f\n", $bid) : ""), sprintf("$Watchpricechange_text: $currency %.2f / %s", $item_info{'current'}, $item_info{'buyer'}));
						}
						bidvaliditycheck(%item_info);
						$watchprice = $item_info{'current'};
					}
                                        else
                                        {
                                                if ($Watch && $zealous)
                                                {
                                                        my $txt = get_zealous_text(%item_info);
                                                        if ($txt ne $zealous_text)
                                                        {
                                                                print "\n" if (! $quieter);
                                                                log_message(checkitem(%item_info) . (($Track == -1) ? sprintf("\n\nReminder: Your maximum bid for this item is $currency %.2f\n", $bid) : ""), "Status change");
                                                                $zealous_text = $txt;
                                                        }
                                                }
                                        }
				}
			}
                        $watchbadhttp = 0;
                        $watchchecktime = watchchecktime_next();
                }

		# Sleep for 1 second
                print "\r$clearline$biddingin_text: " . seconds2time ($end_time - time() - $gap) . " seconds" if (! $quieter);
		$atomictime = time();
                sleep (1);
        }
        print "\n" if (! $quieter);
        $| = $flush_state;
}
silentoff();

Track_print(sprintf("Bidding $currency %.2f for item $item ($item_name)\n", $bid));
Track_log_message(bid($item, $bid, $user, $pass), sprintf("Bidding $currency %.2f", $bid));

$flush_state = $|;
while (time() < ($end_time + $acheck))
{
        $| = 1;

        print "\r${clearline}Checking winner in: " . seconds2time ($end_time + $acheck - time()) . " seconds" if (! $quieter);
        sleep (1);
}
print "\n" if ($quieter);
$| = $flush_state;

my $insurebidend = 1;
while ($insurebidend == 1)
{
        $res = item_check($item);
        checkbadhttp_fatal(". Could not retrieve item information after bid, please check manualy");
        %item_info = extractii($res->content);
        if ($item_info{'left'} !~ /^$Exitstatus_checktext/)
        {
                print "Sleeping some additional 10 seconds, seems the bid is not fully ended[" . $item_info{'left'} ."]\n";
                sleep(10);
        }
        else
        {
                $insurebidend = 0;
        }
}

my $tmp_winner = $item_info{'buyer'};
$tmp_winner =~ s/^Private\sAuction$/Unknown Winner/;
log_message(checkitem(%item_info), sprintf("End of bid: $currency %.2f / %s", $item_info{'current'}, $tmp_winner));

bidend_exit(%item_info);

## END Main
############################################################
## Sub functions

sub title
{
	print <<EOF;
eBay Agent Version $version
Copyright 2001-2004 Andrew Wansink & Martial Michel
This software comes with NO WARRANTY, $0 -D for details
Type "perldoc $0" for documentation
EOF
}

##########

sub disclaimer
{
	print <<EOF;
                            NO WARRANTY

  11. BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  EXCEPT WHEN
OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS
TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE
PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
REPAIR OR CORRECTION.

  12. IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES,
INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING
OUT OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED
TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY
YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER
PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE
POSSIBILITY OF SUCH DAMAGES.

EOF
	
	ok_exit();
}

##########

sub usage
{
	print <<EOF;

* eBayAgent website located at: http://sourceforge.net/projects/ebayagent/
* Please send bug report or feature request to: martial\@users.sourceforge.net
* Updates to script are ususaly made to CVS repository before they appear in a
full release. Please check CVS for fixes/updates (see website).

Usage: $0 [-D] [-v] [-q] [-k] -i item [-u user -p pass] [-l country] [-m [-M email]] [-S [smtpserver]] [-P [proxy]] [-f useragent] [-r rcfile] [-c] [-t] [-b bid [-s [0]] [-g gap] [-d time] [-a time] [-N] [-h retry -H time] [-n] [-w time | -W time] [-B] [-E] [-A]] [-T time -z] [-R]\n

EOF

	$keypress = 0;

	ok_exit();
}

##########

sub log_message
{
	my $message = shift @_;
	my $subjectadd = shift @_;

	chomp ($message);
	
	$message = "[" . scalar (localtime) . "] " . $message;
	
	print $message, "\n";
	
	my $subject = "[eBayAgent]";
	$subject .= " $subjectadd |" if ($subjectadd !~ /^\s*$/);
	$subject .= " $item: $item_name";

	if ((! $check) && $mail)
	{
                if ($Sendmail != -1)
                {
                        my %mail =
                                (
                                 To      => $Mailuser,
                                 From    => $Mailuser,
                                 Subject => $subject,
                                 Message => "$message\n"
                                );
                        sendmail(%mail) or warn "Mail::Sendmail error: " . $Mail::Sendmail::error;
                }
                else
                {
                        open (SENDMAIL, "|/usr/sbin/sendmail -t");
                        print SENDMAIL "To: $Mailuser\nSubject: $subject\n\n$message\n";
                        close (SENDMAIL);
                }
	}
}

##########

sub seconds2time
{
	my $time = shift;
	
	my $seconds = $time % 60;
	my $minutes = int (($time / 60) % 60);
	my $hours   = int (($time / 3600) % 24);
	my $days    = int ($time / 86400);
	my $daysadd = ($days > 1) ? "days" : (($days > 0) ? "day" : "");

	return (($daysadd ne "") ? "$days $daysadd, " : "")
	  . sprintf("%02d:%02d:%02d", $hours, $minutes, $seconds);
}

##########

sub bid
{
	return if ($Track != -1);

	my $item   = shift @_;
	my $bid    = shift @_;
	my $user   = shift @_;
	my $pass   = shift @_;

	my $ubid = bidvalue_locale($bid);
	my $ubid2 = bidvalue2_locale($bid);

        $user = uri_escape($user);
        $pass = uri_escape($pass);

        my ($bidurl, $bidurl2);
	if ($iseBayMotors == 0)
	{
                $bidurl  = ($item_bidurl ne "") ? 
                        $item_bidurl : $item_bid_baseurl;
		$bidurl2 = ($item_bidurl2 ne "") ? 
                        $item_bidurl2 : $item_bid_baseurl2;
        }
        else
        {
                $bidurl  = $eBM_item_bid_baseurl;
                $bidurl2 = $bidurl;
	}

	# do an localised request either, to enforce bidding key creation
	if ($locale ne "com")
	{
		my $url = "http://cgi.ebay.de/ws/eBayISAPI.dll?ViewItem&item=" . $item;
		my $res = getpage($url);
		if (checkbadhttp())
		{
			return $res;
		}
	    
	}

	my $req;
	my $post = &bidpost1_locale($item, $ubid);
        dprint("BID(1/3): ", crypturl($post));
	my $res = getpage($bidurl . "?" . $post);
	
	checkbadhttp_bid();

	my ($tmp, %all) = getallhiddeninput($res, "SignInWelcome");

        error_exit("Could not get 'Referring URL' information, quitting\n")
                if (! exists $all{"ru"});

        $all{"ru"} =~ s/\&amp\;/&/g;

	$req = new HTTP::Request POST => $item_bid_baseurl2;
	$req->content_type ('application/x-www-form-urlencoded');
		
	$post = "MfcISAPICommand=SignInWelcome&siteid=77&co_partnerId=2&UsingSSL=1&ru=" . uri_escape($all{"ru"}) . "&pp=&pa1=&pa2=&pa3=&i1=-1&pageType=2142&userid=$user&pass=$pass";
        dprint("BID(2/3): ", crypturl($post));
	$req->content ($post);
	$res = response($req);
	checkbadhttp_bid();

	$post = "http://offer.ebay.de/ws/eBayISAPI.dll?MfcISAPICommand=MakeBid&fb=2&co_partnerid=&item=" . $item . "&maxbid=" . $ubid2 . "&BidBinInfo=<bidInfo><ID>" . $item . "</ID><MB>" . $ubid2 . "</MB><QTY></QTY></bidInfo>";
	$res = getpage($post);
	if (checkbadhttp())
	{
		return $res;
	}
	    
	# Bidding key extraction (2 possibilities)
	my $res_tmp = uri_unescape($res->content);
#	# In Germany bidding key is coded in 'Referring URL', "ru"
#	if (locale ne "com")
#	{
#		$res_tmp = $all{"ru"};
#	}
#	my $tkey = "";
#	if ($res_tmp =~ m/"key" value="(.*?)"/)
#        {
#                $tkey = $1;
#	}
#	elsif ($res_tmp =~ m%<key>([^<]+)</key>%i)
#	{
#                $tkey = $1;
#	}
#
#	if ($tkey eq "")
#        {
#                error_print("Was not able to get bidding key, aborting (sorry)");
#                error_exit();
#        }

        ($tmp, %all) = 
          getallhiddeninput($res, $iseBayMotors ? "Confirm Bid" : "MakeBid");

        error_exit("Could not get 'bidding key' information, quitting\n")
                if (! exists $all{"key"});
        my $tkey = $all{"key"};
        error_exit("Could not get 'New password' information, quitting\n")
                if (! exists $all{"pass"});

        if ($debug_donotbid)
        {
                dprint("SKIPPING BIDDING PROCESS (Bid key: \'$tkey\')\n");
        }
        else
        {
                dprint("Bid key: \'$tkey\'");
                $pass = $all{"pass"};
                dprint("New password: \'$pass\'");
                my $req = new HTTP::Request POST => $item_bid_baseurl3;
                $req->content_type ('application/x-www-form-urlencoded');
                $post = &bidpost2_locale($item, $tkey, $ubid, $user, $pass);
                dprint("BID(3/3): ", crypturl($post));
                $req->content ($post);
                $res = response($req);
                checkbadhttp_bid();
        }
	
	my $server_time = "";
	
	$server_time = $res->header ('date');
	$server_time = "Placed bid at server time: $server_time\n";
	
	return $server_time;
}

##########

sub buyitnow
{
	my $item   = shift @_;
	my $bid    = shift @_;
	my $user   = shift @_;
	my $pass   = shift @_;

        $user = uri_escape($user);
        $pass = uri_escape($pass);

	my $req;
	$req = new HTTP::Request POST =>'http://cgi.ebay.com/aw-cgi/eBayISAPI.dll';
	$req->content_type ('application/x-www-form-urlencoded');
	$req->content ("MfcISAPICommand=BinConfirm&item=$item&quantity=1");
	
	my $res = response($req);
	checkbadhttp_bid();

        my ($tmp, %all) = 
          getallhiddeninput($res, $iseBayMotors ? "Commit to Buy" : "Submit");

        error_exit("Could not get 'Referring URL' information, quitting\n")
                if (! exists $all{"ru"});

        $all{"ru"} =~ s/\&amp\;/&/g;

        if ($debug_donotbid)
        {
                dprint("SKIPPING BIDDING PROCESS\n");
        }
        else
        {
		my $req;
		$req = new HTTP::Request POST =>'http://signin.ebay.com/aw-cgi/eBayISAPI.dll';
		$req->content_type ('application/x-www-form-urlencoded');
		
		my $post;
		$post = "MfcISAPICommand=SignInWelcome&item=$item&Quantity=1&javascriptenabled=0&UsingSSL=0&mode=1&userid=$user&pass=$pass&ru=" . uri_escape($all{"ru"});
		$req->content ($post);
		$res = response($req);
		checkbadhttp_bid();
	}
	
	my $server_time = "";
	
	$server_time = $res->header ('date');
	$server_time = "Placed \'Buy It Now\' with eBay at server time: $server_time\n";
	
	return $server_time;
}


##########

sub lefttime2string
{ # Convert seconds left to a printable string
        my $left = shift @_;
        my $result = "";
        my $two == 0;

        my $days = int ($left / (3600*24));
        if ($days > 0)
        {
                $result .= "$days day" . (($days > 1) ? "s " : " ");
                $two++;
        }

        $left -= $days * (3600*24);
        my $hours = int ($left / 3600);
        if (($hours > 0) || ($days > 0))
        {
                $result .= "$hours hour" . (($hours > 1) ? "s " : " ");
                $two++;
        }
        return ($result) if ($two > 1);

        $left -= $hours * 3600;
        my $minutes = int ($left / 60);
        if (($minutes > 0) || ($hours > 0))
        {
                $result .= "$minutes minute" . (($minutes > 1) ? "s " : " ");
                $two++;
        }
        return ($result) if ($two > 1);

        $left -= $minutes * 60;
        $result .=" $left second" . (($left > 1) ? "s" : "");

        return ($result);
}

sub string2time
{ # Converts eBay end time strings into usable values
        my $iend = shift @_;
        my $result;
        my $end;

        # Clean up
        $iend =~ s/^\W+//;
        $iend =~ s/\W+$//;

        my ($mon, $day, $year, $daytime) =
                $iend =~ m!^\s*([A-Za-z]{3})-([0-9]{2})-([0-9]{2})\W+?(.*)$!;

        # Workaround for bug in Date::Parse module, doesn't seem to
        # recognise daylight savings zones when no comma is present

        $daytime =~ m/^([0-9:]{5,8}) (.*)$/;
        if ($2 eq "PDT")
        {
                $end = "$day $mon $year $1 -0700";
        }
        else
        {
                $end = "$day $mon $year $daytime";
        }

        $result = str2time ($end);

        print("[$iend] -> [$end] -> [$result]\n") if ($debug);

        return($result);
}

sub end_time
{
	my $res = shift @_;

	my $ctime = time();

	return $ntponly_endtime if ($ntponly && $ntponly_endtime);
	
	# Grab the server time from the "date" header and convert to unix time
	my ($date) = $res->header ('date') =~ m/^(.*? GMT)/;
	my $time = str2time ($date);
	
	if (! $silent)
	{
		print "Current date (eBay)       : ". gmtime($time) ." (GMT)\n";
		print "Current date (this system): ". gmtime($ctime) ." (GMT)\n";
	}
	my $end;
	
	# Grab the auction end time from the items web page and convert to
	# unix time
	my $res_tmp = uri_unescape($res->content);
	if ($res_tmp =~ m!\(Ends.+?(.*?)\)!is)
	{
		$end = $1;
		$end =~ s/[\n\r]//gs;
		
		$end = string2time($end);
	}
	else
	{
                if ($watchbadhttp == 1)
                { # We are in a watch item check loop ... do _not_ exit !
                        log_message ("Unable to determine eBay end time", "Warning");
                        return ($watchbadclockreturn);
                }

                log_message ("Unable to determine eBay end time", "Error");
                error_exit();
	}
	
	if (! $silent)
	{
		print "Bid End time (eBay)       : ". gmtime($end) . " (GMT)\n";
	}

	my $ttime = $end;

	if ($ntponly)
	{
                $ntponly_endtime = $ttime;
        }
	else
        {
                $ttime = $ttime - $time + time();
        }

	return $ttime;
}

##########

sub watchchecktime_next
{
	my $atomictime = time();
	my $tmp_endtime = $end_time - $gap - $dcheck;

	if ($watchtimelap)
	{ # W/watch is active
		my $tmp = $atomictime + 60*$watchtimelap;
		
		# 5 min check ?
		if (($tmp > ($tmp_endtime - 300))
		    && ($atomictime < ($tmp_endtime - 300)))
		{
			$tmp = $tmp_endtime - 300;
			return $tmp;
		}
		
		# are we in the 1min check range ?
		if (($tmp > ($tmp_endtime - 60))
		    && ($atomictime > ($tmp_endtime - 300))
		    && ($watchchecktime != ($tmp_endtime - 60)))
		{
			$tmp = $tmp_endtime - 60;
			return $tmp;
		}

		return $tmp;
	}
	else
	{
		# W/watch is not active, so we are here because we just
		# did the 5 min check ... let us prepare for the 1 min check
		$watch_msg = "1min time synch mode";
		return ($tmp_endtime - 60);
	}
}

##########

sub item_name
{
	my $res = shift @_;
	
	my $tmp = $res->as_string;
	chomp $tmp;
	my $ret = $1 if ($tmp =~ m%<title>\s*(.*?)\s*</title>%ism);
	$ret =~ s/^eBay item \d+ \(Ends\s+?(.*?)\)\s\-\s+?//;
	$ret =~ s%^.*\n%%gm;
	
	return $ret;
}

##########

sub set_useragent
{
	$useragent = new LWP::UserAgent;

	if ($fake_useragent ne "")
	{
		$useragent->agent($fake_useragent);
	}
	else
	{
		$useragent->agent("eBay Agent/$version " . $useragent->agent);
	}

	if ($Proxy ne "-1")
	{
		if (($Proxy eq "") || ($Proxy eq "1"))
		{
			$useragent->env_proxy();
		}
		else
		{
			$useragent->proxy('http', $Proxy);
		}
	}
	
	## Let us ready the Cookie Jar
	$cjar = ($debug > 2)
		? HTTP::Cookies->new(File => $cjar_file,
				     AutoSave => 1,
				     hide_cookie2 => 1)
			: HTTP::Cookies->new(hide_cookie2 => 1);
	$useragent->cookie_jar($cjar);
}

##########

sub getpage
{
	my $url = shift @_;
	
	my $req = new HTTP::Request GET => $url;

	my $res = response($req);

	return $res;
}

##########

sub response
{ # All HTTP requests must go through here
	my $request = shift @_;
	
	print "${lineskip}Requesting: " . crypturl($request->url()) . "\n"
                if (($debug > 1) || (! $silent));
	
        # Going via this function _always_ add/extract the cookies
        $cjar->add_cookie_header($request);

	my $response = $useragent->request ($request);

	if ($debug > 1)
	{
                writedebugfile("formated web page content",
                  $response->as_string);
	}

        if ($response->is_redirect())
        {
                # should this be reached? 
                # as $useragent->request() is supposed to handle redirects.
                # it is reached when the useragent detects a redirect loop.
                # when else? 
                $cjar->extract_cookies($response);
                dprint("[Cookies:\n" . $cjar->as_string() ."]") if ($debug > 2);

                $response->as_string =~ m%[lL]ocation:\s+(.*)%g;
                return getpage($1);
        }

	if (! $response->is_success())
	{
                if ($watchbadhttp == 1)
                {
                        Track_log_message ("Warning: Bad response from eBay for requested page (" . crypturl($request->url()) . ") while in " . $watch_msg, $watch_msg . " warning");
                        $watchbadhttp = -1;
                        $httpretry_inc = 0;
                        return($response);
                }
                else
                {
                        log_message ("Error: Bad response from eBay for requested page (" . crypturl($request->url()) . ")" , "Web access error");
                        error_exit()
                }
	}

	$cjar->extract_cookies($response);
        dprint("[Cookies:\n" . $cjar->as_string() ."]") if ($debug > 2);

	# Supposedely here we have a "success" entry, but it seems that eBay
	# at times returns a success value but no content (Many thanks to
	# Klaus Singvogel for reporting this strange behavior), so ...
        # 20030121 -- MM : added a multiple line check (some eBay pages
        #  seem to start with <script language="JavaScript">
	my $res_tmp = uri_unescape($response->content);
	if ($res_tmp !~ m%^<html[^>]*>%im)
	{
		$httpretry_inc++;
		dprint("No content from eBay ($httpretry_inc/$httpretry)");
		if ($httpretry_inc > $httpretry)
		{
			return $response;
		}
		else
		{
			sleep($Hrdelay);
			return response($request);
		}
	}

	$httpretry_inc = 0;
	return $response;
}

##########

sub timetest
{
	my $url = "http://cgi.ebay.com/aw-cgi/eBayISAPI.dll?TimeShow";

	my $response = getpage($url);
	if (checkbadhttp())
	{
		error_exit($httpretry_msg, " : Exiting\n");
	}

	my $res_tmp = uri_unescape($response->content);
	if ($res_tmp =~ m%The official eBay Time is now:</p><p><b>(.*?)<%)
#At the tone, the time will be... <b>(.*?)</)
	{
		log_message ("eBay's web page says the time is: $1", "Time Check: $1");
	}
	else
	{
		log_message ("Couldn't get time from eBay's timeshow page", "Time Check Error");
		error_exit();
	}
	
	my ($date) = $response->header ('date') =~ m/^(.*? GMT)/;
	log_message ("eBay's web server headers say: $1", "Time Check: $1");
	
	ok_exit();
}

##########

sub item_check
{
        my $item = shift @_;
        my $url = $item_check_baseurl . $item;
	
        my $res = getpage($url);
	if (checkbadhttp())
	{
		return $res;
	}

        my $oldsilent = silentstatus();
        silenton();
        if (adultcheck($res))
        {
                if ($Adult)
                {
                        adultlogin($res);
                        $res = getpage($url);
			if (checkbadhttp())
			{
				return $res;
			}
                }
                else
                {
                        error_print("Can not bid for an \"Adult\" item without the \"Adult\" option enabled");
                        error_exit();
                }
        }
        silentset($oldsilent);

        return $res;
}

#########

sub writedebugfile
{
        my $debugfilelabel = shift @_;
	my $debugfile = "/tmp/eBayAgent-$$-" . $debugfilecounter++;

	open FILE, ">$debugfile"
          or error_exit("Error open file $debugfile for $debugfilelabel: $!\n");

        dprint( $debugfilelabel . ': ' . $debugfile );

	print FILE @_;
	close FILE;
}

##########

sub checkbadhttp
{
	return ($httpretry_inc > $httpretry);
}

sub checkbadhttp_bid
{
	if (checkbadhttp())
	{
		log_message($httpretry_msg .
			    " : no choice left but to cancel bid, sorry\n",
			   $httpretry_hmsg);
		error_exit();
	}
}

sub checkbadhttp_Adult
{
	if (checkbadhttp())
	{
		log_message(concat_strings($httpretry_msg,
					   @_, " : Exiting\n"),
			    $httpretry_hmsg);
		error_exit();
	}
}

sub checkbadhttp_fatal
{
	if (checkbadhttp())
	{
		log_message(concat_strings("ERROR: ",
					   $httpretry_msg, @_, "\n"),
			    $httpretry_hmsg);
		error_exit();
	}
}
		
##########
# "Adult" part

sub adultcheck
{
        my $res = shift @_;
        $res = cleanpage($res->content);

        if ($res =~ m%eBayISAPI\.dll\?AdultLoginShow%mg)
        {
                return 1;
        }

        return 0;
}


sub adultlogin
{
        # Note to work on this: All "adult items" are now under the
        # "Everything Else > Mature Audiences" section
        my $res = shift @_;


        ## "Adult Login" is a 3 phases (web pages) process
        my $EAD = # Error Add Text
                "\n\nAre you registered with eBay as a person that can access to the\n\"Mature Audiences Category\" ?\n\nPlease check eBayAgent's man/perldoc content in the \"--Adult\"\noption section for more informations on what is required in order\nto make this option work.\n";

        ########## Page 1: "Adult Login"
        my ($tmp1, %all1) = getallhiddeninput($res, "SignInWelcome");
        if ($all1{'MfcISAPICommand'} ne "SignInWelcome")
        {
                error_exit("Bad Adult FORM content (Page 1/3)" . $EAD);
        }
        if ($tmp1 !~ m/form method=\"?post\"?.*? action=\"(\S+?)\"/mi)
        {
                error_exit("Bad Adult URL from eBay (Page 1/3)" . $EAD);
        }
        my $Purl1 = $1;

        my $user = uri_escape($user);
        my $pass = uri_escape($pass);

        my $content1 = "MfcISAPICommand=SignInWelcome&userid=$user&pass=$pass";
        delete $all1{'MfcISAPICommand'};
        foreach my $key1 (keys %all1)
        {
                my $tmp = $all1{$key1};
                while ($tmp =~ s/\&amp\;/&/) {1;}
                $tmp = uri_escape($tmp);
                $content1 .= "&${key1}=$tmp";
        }
        my $req1 = new HTTP::Request POST => $Purl1;
	$req1->content($content1);
        my $uri1 = $res->base;
        $req1->referer($uri1->as_string);
        my $res1 = response($req1);
	checkbadhttp_Adult(". Adult (Page 1/3)");

        ########## Page 2: "Terms of Use / Mature Audiences Category"
        my ($tmp2, %all2) = getallhiddeninput($res1, "AdultLogin");
        if ($all2{'MfcISAPICommand'} ne "AdultLogin")
        {
                error_exit("Bad Adult FORM content (Page 2/3)" . $EAD);
        }
        if ($tmp2 !~ m/form method=\"?post\"? action=\"(\S+?)\"/mi) 
        {
                error_exit("Bad Adult URL from eBay (Page 2/3)" . $EAD);
        }
        my $Purl2 = $1;

        my $content2 = "MfcISAPICommand=AdultLogin";
        delete $all2{'MfcISAPICommand'};
        foreach my $key2 (keys %all2)
        {
                my $tmp = $all2{$key2};
                while ($tmp =~ s/\&amp\;/&/) {1;}
                $tmp = uri_escape($tmp);
                $content2 .= "&${key2}=$tmp";
        }
        my $req2 = new HTTP::Request POST => $Purl2;
	$req2->content($content2);
        my $uri2 = $res1->base;
        $req2->referer($uri2->as_string);
        my $res2 = response($req2);
	checkbadhttp_Adult(". Adult (Page 2/3)");

        ########## Page 3: "Your information has been verified"
        my $tmp3 = cleanpage($res2->content());
        if ($tmp3 !~ m/Your information has been verified/i) {
                error_exit("Wrong Adult Identification result (Page 3/3)"
                           . $EAD);
        }
}

sub getallhiddeninput
{ # Get all "input type=hidden" entry in the form
        my $res = shift @_;
        my $limit = shift @_;
        my %allhidden;

        my $tmp = uri_unescape($res->content);
        my $tmp2 = "";
        while (($tmp2 eq "") && ($tmp =~ s/(\<form.+?\/form\>)//is))
        {
                $tmp2 = $1;
                if ($tmp2 !~ m/value\=\"?$limit/i)
                {
                        $tmp2 = "";
                        next;
                }
        }
        if ($tmp2 eq "") {
                error_exit("Did not find requested POST Command ($limit)\n");
        }

        while (
               ($tmp2 =~ s/input\s+type=\"?hidden\"?\s+name=\"?(\S+?)\"?\s+value=\"([^\"]+?)\"\>//i)
	       ||
               ($tmp2 =~ s/input\s+type=\"?hidden\"?\s+name=\"?(\S+?)\"?\s+value=\"([^\"]+?)\"\s//i)
               ||
               ($tmp2 =~ s/input\s+type=\"?hidden\"?\s+name=\"?(\S+?)\"?\s+value=\"?(\S*?)\"?\>//i)
              )
        {
                dprint("(FORM) [$1|$2]") if ($debug > 2);
                $allhidden{$1} = $2;
        }

        return ($tmp2, %allhidden);
}

##########

sub extractii
{ # Extract item_info
	# MM: Method adapted from mebay.pl (by Gargola Software)
	my %item_info;
	my $page_content = shift @_;

        $page_content = cleanpage($page_content);
	
#	dprint("--------------------\n", $page_content, "\n-----------------");

	# We re-process the item so let us think we have no specificities yet
	$add_item_info{'counter'} = 0;
	$add_item_info{'buyitnowonly'} = 0;
	$add_item_info{'buyitnow'} = 0;
	$add_item_info{'reserve'} = 0;
	$add_item_info{'buyitnow_approx'} = "";

	# Easy check for new layout
	my $raw_item_name = $1
		if ($page_content =~ m%<title>\s*(.*?)\s*</title>%ism);
        if ($raw_item_name =~ m/^eBay item $item \(Ends\s+?(.*?)\)\s\-\s+?/)
	{
#                dprint("##### Layout > July 2003 #####");
		$isnewLayout = 1;
	}
	else
	{
#                dprint("##### Old Layout #####");
		$isnewLayout = 0;
	}

	# Mostly everything ...
	while ($page_content =~ m,<td[^>]*>\s*(started|ends|currently|current bid|\[buy_it_now\]\s*price|first bid|starting bid|time left|item location|location|country|country\/region|seller \(rating\)|high bid|high bidder|buyer|winning bid|winning bidder|\# of bids)\:?\s*</td>\s*<td[^>]*>(.*?)</td>,igs)
	{
		my ($t1, $t2) = ($1, $2);
		if ($t1 =~ /buy\_it\_now/i)
		{
			if ($isnewLayout == 1)
			{
				$t1 =~ s%buy\_it\_now%buy_it_case_now%i;
                                $skipsubdata = 1;
				my ($a, $b) = formatentryii($t1, $t2);
                                $skipsubdata = 0;
				next;
			}
		}
		my ($a, $b) = formatentryii($t1, $t2);
		next if (exists $item_info{$a});
		dprint("AddMainInfo [$a|$b]");
		$item_info{$a} = $b;
	}

	# Positive feedback Percentage extraction
        if ($page_content =~ m!<td[^>]*>.*positive\s(?:feedback|bewertung(?:en))\:\s*([\d,.]+\%)!is)
        {
                my ($a, $b) = formatentryii("posfeedperc", $1);
                $item_info{$a} = $b;
        }

	
	# Buy It Now (new layout / includes approx)
	if ($page_content =~ m,<td[^>]*>\s*(\[buy\_it\_now\]\s*</td><td[^>]*>\s*price)\:?\s*</td></tr></table></td><td[^>]*>(.*?)</td>,igs)
	{
		my ($t1, $t2) = ($1, $2);
		$t1 = cleanhtmltags($t1);
		if ($t1 =~ /buy\_it\_now/i)
		{
			$t1 =~ s%buy\_it\_now%buy_it_case_now%i;
                        $skipsubdata = 1;
			my ($a, $b) = formatentryii($t1, $t2);
                        $skipsubdata = 0;
		}
	}

	# 'Buy It Now' text (outside of the normal "box")
	if (($add_item_info{'buyitnow'} == 0)
	    && ($page_content =~ m,<td[^>]*>\s*(buy\s+it\s+now\s+for.*?)</tr>,is))
	{
		# We fake a request to set the special triggers
		# and drop the result
		my $tmp_t = "$currency 0.99 " . $1;
		my ($a, $b) = formatentryii("first bid", $tmp_t);
	}

        # Start / End time (new layout)
        if ($page_content =~ m,(Start Time):\s*(.*?)\Q$cutchar\E\s*(End Time):\s*(.*?)<p>,is)
        {
                my ($t1, $t2, $u1, $u2) = ($1, $2, $3, $4);

                my $v = string2time($t2);
                $v = localtime($v);
                my ($a, $b) = formatentryii($t1, $v);
		if (! exists $item_info{$a})
                {
                        dprint("AddMainInfo [$a|$b]");
                        $item_info{$a} = $b;
                }

                ($a, $b, $v) = ("", "", "");
                $v = string2time($u2);
                $v = localtime($v);
                ($a, $b) = formatentryii($u1, $v);
		if (! exists $item_info{$a})
                {
                        dprint("AddMainInfo [$a|$b]");
                        $item_info{$a} = $b;
                }
        }

	# Bid increment (old layout)
	#  (It seems that eBay changes the layout for that info, so  we get
	#  everything until the end of the line (</tr>) and parse it for
	#  the currency of the auction)
	if ($page_content =~ m,<td[^>]*>\s*(bid increment\:)(.*?)</tr>,is)
	{
		my ($tmp_a, $tmp_b) = ($1, $2);
		$tmp_b =~ s,^.*?</td>,,i;
		my $tmp_c = "";
		do {
			if ($tmp_b =~ s,<td[^>]*>(.*?)</td>,,i)
			{
				my $tmp_d = $1;
				if ($tmp_d =~ m/\Q$currency/) {
					$tmp_c = $tmp_d;
				}
			}
		} until ($tmp_c ne "" || $tmp_b eq "");
		my ($a, $b) = formatentryii($tmp_a, $tmp_c);
		$item_info{$a} = $b;
	}

	# Minimum bid (old layout)
	if ($page_content =~ m,<td[^>]*>\s*\((minimum bid\:)(.+?)\)\s*</td>,igs)
	{
		my ($a, $b) = formatentryii($1, $2);
		$item_info{$a} = $b;
	}

        # Number of bids + starting bid (layout July 2003)
        if (! exists $item_info{'bids'})
        {
                if ($page_content =~ m,<td[^>]*>\s*(history)\s*\:?\s*</td>\s*<td[^>]*>(.+?)</td>,is)
                {
                        my ($a, $b, $c) = ($1, $2, $2);
                        $b =~ m%(\d+)\s+bids?%;
                        ($a, $b) = formatentryii($a, $1);
                        $item_info{$a} = $b;

			if ($c =~ m%\((.+?)\s(starting bid)\)%)
			{
				($a, $b) = ($2, $1);
				($a, $b) = formatentryii($a, $b);
				if (! exists $item_info{'initbid'})
				{
					$item_info{$a} = $b;
				}
			}
                }
        }

	# Minimum bid (layout July 2003)
        if (! exists $item_info{'minbid'})
        {
                if ($page_content =~ m,<td[^>]*>$s_re*\(Enter$s_re+(.+?)$s_re+or more\)\s*</td>,is)
                {
                        my ($a, $b) = formatentryii('minbid', $1);
                        $item_info{$a} = $b;
                }
        }

        # Seller (layout July 2003)
        if (! exists $item_info{'seller'})
        {
                if ($page_content =~ m,(seller information.*?)</table>,is)
                {
                        my $tmp = cleanhtmltags($1);
                        $tmp =~ s/^seller information//i;
                        $tmp =~ s/(\)).+$/$1/;
                        my ($a, $b) = formatentryii('seller', $tmp);
                        $item_info{$a} = $b;
                }
        }

        # Current (layout July 2003)
        if ((! exists $item_info{'current'}) && (exists $item_info{'initbid'}))
        {
                my ($a, $b) = formatentryii('current', $item_info{'initbid'});
                $item_info{$a} = $b;
        }

        # Bid inc (layout July 2003)
        if (! exists $item_info{'bidinc'}) {
                if (defined $item_info{'minbid'} && defined $item_info{'current'} &&
                    ($item_info{'minbid'} > $item_info{'current'})) 
                {
                        my ($a, $b) = formatentryii('bidinc',
                                                    $item_info{'minbid'} - $item_info{'current'});
                        $item_info{$a} = $b;
                }
        }

        # Country (layout July 2003)
        if ((! exists $item_info{'country'}) && (exists $item_info{'location'}))
        {
                my @tmp = split(/\Q$cutchar\E/, $item_info{'location'});
                if (scalar @tmp == 2)
                {
                        my ($a, $b) = formatentryii('location', $tmp[0]);
                        $item_info{$a} = $b;
                        ($a, $b) = formatentryii('country', $tmp[1]);
                        $item_info{$a} = $b;
                }
        }

	# Clean 'left' and set 'start' and 'end' if not set (new layout)
        if ($raw_item_name =~ m/^eBay item $item \(Ends\s+?(.*?)\)\s\-\s+?/)
        {
                # The new layout does not contain info about the bid start
                # time, we have to calculate it
                my $end_time = $1;
                my $end = string2time($end_time);
                my $start = $end;
                my @tmp = ();

                if (defined $item_info{'left'})
                {
                        $item_info{'left'} =~ s/\Q$cutchar\E\s+add\s+to\s+calendar//i;
                        @tmp = split(/\Q$cutchar\E/, $item_info{'left'})
                }

                if (scalar @tmp == 3)
                {
                        # First line : real time left
                        my ($a, $b) = formatentryii('time left', $tmp[0]);
                        $item_info{$a} = $b;
                        # Second line : listing time information
                        if ($tmp[1] =~ m%(\d+)\-day\slisting%)
                        {
                                $start -= $1*3600*24;
                        }
                }
                else
                {
                        # Some multiple quantity items or buy it now only
                        # items mess up 'left', so we do an additional check
                        # to see if the end time is in the future
                        my $atomictime = time();
                        if ($end > $atomictime)
                        {
                                my $left = $end - $atomictime;
                                $left = lefttime2string($left);
                                my ($a, $b) = formatentryii('time left', $left);
                                if (! exists $item_info{$a})
                                {
                                        $item_info{$a} = $b;
                                }
                        }
                }

                my $txt = "";
                $txt = localtime($start);
                my ($a, $b) = formatentryii('started', $txt);
		if (! exists $item_info{$a})
                {
                        dprint("AddMainInfo [$a|$b]");
                        $item_info{$a} = $b;
                }

                $txt = "";
                $txt = localtime($end);
                ($a, $b) = formatentryii('ends', $txt);
		if (! exists $item_info{$a})
                {
                        dprint("AddMainInfo [$a|$b]");
                        $item_info{$a} = $b;
                }
        }

	# Buy It Now only items check (new layout)
	if ($isnewLayout == 1)
	{
		if ((! exists $item_info{'minbid'})
		    && (! exists $item_info{'bidinc'})
		    && (exists $item_info{'left'}))
		{
			my ($a, $b) = formatentryii("price", "buy_it_now $currency " . $add_item_info{'buyitnow'});
			$item_info{$a} = $b;
		}
	}

	# Auction has ended (new layout)
	if ($isnewLayout == 1)
	{
		if (! exists $item_info{'left'})
		{
			my ($a, $b) = formatentryii("time left", $Exitstatus_checktext);
			$item_info{$a} =  $b;
		}
	}

        # Offer url (.com locale)
        if (($item_bidurl eq "") && ($page_content =~ m#<a\s+href\s*\=\s*\"([^\"]+?offer\.[^\"]+?ebayisapi\.dll)#is))
        {
                $item_bidurl = $1;
                check_eBM( $item_bidurl );
        }

	# Signin url (.com locale)
	if (($item_bidurl2 eq "") && ($page_content =~ m#<form\s+method\s*\=\s*\"get\"\s+action\s*\=\s*\"([^\"]*?signin[^\"]+?)\"#i))
	{
                $item_bidurl2 = $1;
                check_eBM( $item_bidurl2 );
	}

	# a little post processing
	$item_info{'buyer'} =~ s/^private auction-- bidders identities protected$/Private Auction/i;
	$item_info{'buyer'} =~ s/user id kept private/Private Auction/i;
	$item_info{'bids'} =~ s/ bid history\s*$//i;
	$item_info{'bids'} =~ s/\s*\(Bid history\)//i;
	
	if ($debug > 1)
	{
		print "------------------------------";
		print checkitem(%item_info);
		print "------------------------------\n";
	}
	return %item_info;
}

##########

sub formatentryii
{
	my ($a, $b) = @_;

	dprint("RawMainInfo [$a|$b]");
	
	my %entryhash =
		(
		 'started'         => 'start',
		 'start time'      => 'start',
		 'ends'            => 'end',
		 'end time'        => 'end',
                 'current'         => 'current',
		 'currently'       => 'current',
		 'current bid'     => 'current',
		 'price'           => 'current',
		 'winning bid'     => 'current',
		 'first bid'       => 'initbid',
		 'starting bid'    => 'initbid',
		 'time left'       => 'left',
		 'item location'   => 'location',
		 'location'        => 'location',
		 'country'         => 'country',
		 'country/region'  => 'country',
		 'seller'          => 'seller',
		 'seller (rating)' => 'seller',
                 'posfeedperc'     => 'posfeedperc',
		 'high bid'        => 'buyer',
		 'high bidder'     => 'buyer',
                 'winning bidder'  => 'buyer',
                 'buyer'           => 'buyer',
		 '# of bids'       => 'bids',
                 'history'         => 'bids',
                 'bidinc'          => 'bidinc',
		 'bid increment:'  => 'bidinc',
                 'minbid'          => 'minbid',
		 'minimum bid:'    => 'minbid'
		);
	
	my @values = ( 'current', 'initbid', 'bidinc', 'minbid', 'price');
	
	$a = lc $a;
	$b .= $1 if ($a =~ s%^\s*(\[[^\]]+\])\s*%%);
	dprint("Strange key ($a) to add for item_info")
		if (! exists $entryhash{$a});
	$a = $entryhash{$a};

        # check for ebay motors url in content chunk prior to cleaning it.
        # lately only buyer seems to have it. ( 09/28/04 )
        # but we check the old ones anyhow.
	check_eBM($b) if (grep /^$a$/, 'seller', 'current', 'bids' , 'buyer');

	$b =~ s/<[^>]+>//g;
	$b = cleantext($b);
	$b = cleantext(extractsubinfo($a, $b)) if (grep /^$a$/, @values);
	
	return ($a, $b);
}

##########

sub extractsubinfo
{
	my ($a, $b) = @_;
	my $value;
	my $tmp_b;

	dprint("RawSubInfos [$a|$b]");

	$add_item_info{"raw_$a"} = $b;

	# Approx (new layout)
	$b =~ s/Approximately/Approx./is;
	$b =~ s/^(.*?)(\s*Approx\. .*?[\d|\,|\.]+\s*)(.*?)$/$1$3$2/is;

	# New layout -> reformat new entry to old layout matches
	if (($b =~ s/buy\_it(\_case)?\_now//)
	    && ($a eq 'current' || $a eq 'initbid'))
	{
		if ($1 eq "_case")
		{
			$b .= "Buy It Now for " . $b;
		} else {
			$b = "Buy It Now for " . $b;
		}
	}
        if ($b =~ /^Buy\s+It\s+Now\s*?for\s*?(.*?)([\d|\,|\.]+)/)
        { # Item can ONLY be bought using 'Buy It Now'
                $add_item_info{'buyitnowonly'} = 1;
                $b =~ s/^(Buy\s+It\s+Now\s*?for\s*?(.*?[\d|\,|\.]+).*)$/$2/;
                $tmp_b = $1;
        }
	$b =~ s/^(.*?)([\d|\,|\.]+)[\|\s]*//;
	$currency = cleantext($1) if ($currency eq "");
	$value = $2;
	$value =~ s/,//g;

        $b = $tmp_b if ($add_item_info{'buyitnowonly'} == 1);

        $b =~ s%[\(\)]% %g;

        if ($a eq 'current' || $a eq 'initbid')
        {
		# BuyItNow
                if ($b =~ s/Buy\s+It\s+Now\s*?for\s*?(.*?)([\d|\,|\.]+)(.*)$//)
                {
                        my $tmp_value = $2;
			my $tmp_rest = $3;
                        $tmp_value =~ s/,//g;
                        $add_item_info{'buyitnow'} = $tmp_value;
                        $add_item_info{'counter'}++;
			$tmp_rest =~ s/^each//i;
			if ($tmp_rest =~ s/(approx\.\s*?.*?[\d|\,|\.]+)//i)
			{
				$add_item_info{'buyitnow_approx'} = $1;
			}
                        $tmp_rest =~ s/This\s+option\s+disappears\s+once\s+a\s+bid\s+is\s+placed\.//;
			# Recreate an usable string to work with
			$b = $b. $tmp_rest;
                }

		# Reserve (Part 1)
		# Most cases
                if ($b =~ s/reserve\s+not(\s+yet)?\s+met//i)
                {
                        $add_item_info{'reserve'} = 1;
                        $add_item_info{'counter'}++;
                        $b =~ s/This\s+option\s+disappears\s+once\s+the\s+reserve\s+price\s+is\s+met\.//i;
                }
		# Reserve (Part 2)
		# Some broken "block" cases
		if ($b =~ s/This\s+option\s+disappears\s+once\s+the\s+reserve\s+price\s+is\s+met\.//i)
		{
			$add_item_info{'reserve'} = 1;
                        $add_item_info{'counter'}++;
		}
		# Reserve (Part 3)
		# Reserve met case
		if ($b =~ s/reserve\s+met//i)
		{
			$add_item_info{'reserve'} = -1;
			$add_item_info{'counter'}++;
		}
        }
	if ($skipsubdata == 0)
	{
		$add_item_info{"add_$a"} = cleantext_cc($b);
	}

	dprint("LeftSubInfo [" . $add_item_info{"add_$a"} . "]");
	return $value;
}

##########

sub check_eBM
{
        return if ($iseBayMotors != 0);

        my $tmp = shift @_;

        if ($tmp =~ m%ebay\.com/ebaymotors/%)
        {
                $iseBayMotors = 1;
        }
}

##########

sub cleanpage
{
   	my $page_content = shift @_;

	# We have added a call to 'uri_unescape' (so that urls look like urls)
	$page_content = uri_unescape($page_content);

        $page_content =~ s/\t/ /g;
	$page_content =~ s/\n//g;
	$page_content =~ s/\r//g;
	$page_content =~ s/&nbsp;/ /ig;
	$page_content =~ s,</*font.*?>,,ig;
	$page_content =~ s,<br>, ${cutchar} ,ig;
	$page_content =~ s,</*b.*?>,,ig;
	$page_content =~ s,</*img[^>]+alt="buy it now"[^>]*>,[BUY_IT_NOW],ig;
	$page_content =~ s,</*i.*?>,,ig;
	$page_content =~ s,<script.*?</script>,,ig;

	if ($debug > 1)
	{
                writedebugfile("Cleaned content", $page_content);
	}

        return $page_content;
}

sub cleanhtmltags
{
        my $tmp = shift @_;

        $tmp =~ s,<[^>]+>,,g;

        return $tmp;
}

sub cleantext_cc
{
	my $tmp = shift @_;
	
	$tmp =~ s/\Q$cutchar\E//sg;

	return cleantext($tmp);
}

sub cleantext
{
	my $tmp = shift @_;

	my $OK_CHARS='-a-zA-Z0-9_.@ #,:()/\$' .$cutchar;
	$tmp =~ s/[^$OK_CHARS]/ /go;
	while ($tmp =~ s/\s\s/ /g) {1;}
	$tmp =~ s/^\s//g;
	$tmp =~ s/\s$//g;
	
	return $tmp;
}

##########

sub printfii
{ # printf item_info
	my $format = shift @_;
	my $key = shift @_;
	my %item_info = @_;
	my $br = ($debug) ? "** $key **\n" : "";
	
	return $br if (! exists $item_info{$key});
	my $tmp = $item_info{$key};
	$tmp =~ s/^\s+//;
	$tmp =~ s/\s+$//;
	return $br if ($tmp =~ /^$/);

	return sprintf($format, $item_info{$key});
}

sub printaii
{ # print add_item_info
	my $it = shift @_;
	my $res = "";

	if (defined $it && $it !~ /^\s*$/) {
		$it =~ s/^\s+//;
		$it =~ s/\s+$//;
		# 'approx.\s' typo fix
		$it =~ s%(approx\.)(\S)%$1 $2%i;
		$res .= " [$it]";
	}

	return $res;
}

##########

sub checkitem
{
	my %item_info = @_;

        my $tmptmptext = "";
	my $zealtmp = "";

	my $tmptext = "\n";
	$tmptext .= "Item #" . sprintf("%10.0f", $item) . " : " . $item_name . "\n";
        # The one thing always changing in the text is the time, do not add
        # it for zealous comparaisons
        if ($zealous_on == 0) {
                $tmptext .= printfii("Bid start time   : %s\n", 'start', %item_info);
                $tmptext .= printfii("Bid end time     : %s\n", 'end', %item_info);
                $tmptext .= printfii("Time left        : %s\n", 'left', %item_info);
        }

        if ($add_item_info{'buyitnowonly'} == 1)
	{
		my $tshow = 'current';
		$tshow = 'initbid' if (!exists $item_info{'current'});
		$tmptext .= printfii("Price        [*] : $currency %.2f" . printaii($add_item_info{'buyitnow_approx'}) . "\n", $tshow, %item_info);
	}
	else
	{
		$tmptext .= printfii("First bid        : $currency %.2f" . printaii($add_item_info{"add_initbid"}) . "\n", 'initbid', %item_info);
		$tmptext .= printfii("Current price    : $currency %.2f" . printaii($add_item_info{'add_current'}) . "\n", 'current', %item_info);
	}
	
	# Do not report feedback in zealous reports (buyer)
	$zealtmp = printfii("Bidder           : %s\n", 'buyer', %item_info);
	$zealtmp =~ s/\(\d+\)\s*$// 
		if ($zealous_on == 1);
	$tmptext .= $zealtmp;

	$tmptext .= printfii("Bid increment    : $currency %.2f" . printaii($add_item_info{"add_bidinc"}) . "\n", 'bidinc', %item_info);
	$tmptext .= printfii("Minimum bid      : $currency %.2f" . printaii($add_item_info{"add_minbid"}) . "\n", 'minbid', %item_info);
        if ($add_item_info{'buyitnowonly'} == 0)
        {
                $tmptext .= printfii("Number of bids   : %s\n", 'bids', %item_info);
        }

	# Do not report feedback in zealous reports (seller)
        $zealtmp = printfii("Seller           : %s\n", 'seller', %item_info);
	$zealtmp =~ s/\((\d+)\)//g;
	$zealtmp .= sprintf("Seller Feedback  : Total=%s / Positive=%s %%\n", $1, $item_info{'posfeedperc'})
		if ($zealous_on != 1);

	$tmptext .= $zealtmp;	

	$tmptext .= printfii("Location         : %s\n", 'location', %item_info);
	$tmptext .= printfii("Country          : %s\n", 'country', %item_info);

        if ($add_item_info{'counter'} > 0)
        {
                $tmptmptext .= sprintf(" - \'Buy It Now\' is available for $currency %.2f" . printaii($add_item_info{'buyitnow_approx'}) . "\n", $add_item_info{'buyitnow'}) if (($add_item_info{'buyitnowonly'} == 0) && ($add_item_info{'buyitnow'} > 0));
		
		if ($add_item_info{'reserve'} > 0)
		{
			$tmptmptext .= " - Reserve price is not yet met\n";
		}
		elsif ($add_item_info{'reserve'} < 0)
		{
			$tmptmptext .= " - Reserve price is met\n";
		}			
        }

        $tmptmptext .= " - This is an \'eBayMotors\' item\n"
                if ($iseBayMotors != 0);

        $tmptext .= "Additional info  :\n" . $tmptmptext
                if ($tmptmptext ne "");

	if ($add_item_info{'buyitnowonly'} == 1) {
		$tmptext .= "[*] Special Note :\n";
		$tmptext .= " - Item can ONLY be bought using \'Buy It Now\'\n";
	}

	return $tmptext;
}

##########

sub bidvaliditycheck
{
	my %item_info = @_;
	
	if ($end_time - time() < 0)
	{
		log_message ("Bid for item $item ($item_name) finished !", "Bid canceled: Finished");
		error_exit();
	}
	
	if ($item_info{'current'} <= 0)
	{
		log_message(sprintf("Canceling Bid: incoherent current bid price: $currency %.2f (%s) ", $item_info{'current'}, $add_item_info{'raw_current'}), "Bid canceled: inchoherent current bid price");
		error_exit();
	}
	
	if (($Track == -1) && ($bid < $item_info{'minbid'}))
	{
		if ($item_info{'buyer'} =~ /^Private\sAuction$/)
		{
			log_message(sprintf("Warning: this is a \"Private Auction\"; it is impossible to know if your bid of $currency %.2f (which is under the minimum bid amount of $currency %.2f) is/will be the winning bid\n", $bid, $item_info{'minbid'}), sprintf("Warning: can not determine if under the minimum bid of $currency %.2f", $item_info{'minbid'}));
		}
		elsif ($item_info{'buyer'} =~ /^$user\s*\(/)
                {
                        log_message(sprintf("Warning: your bid of $currency %.2f is under the minimum bid of $currency %.2f\n\nNote that the program will not cancel the bid, since you are the current highest bidder", $bid, $item_info{'minbid'}), sprintf("Bid warning: under the minimum bid of $currency %.2f", $item_info{'minbid'}));
                }
                elsif ($debug)
		{
			log_message("Warning: Under the min bid but debug mode '$debug' allows bids under the minimum... continuing", "Debug authorized under minimum amount bid");
		}
		else
                {
                        log_message(sprintf("Canceling bid: your bid of $currency %.2f is under the minimum bid of $currency %.2f", $bid, $item_info{'minbid'}), sprintf("Bid canceled: under the minimum bid of $currency %.2f", $item_info{'minbid'}));
                        bidend_exit(%item_info);
                }
	}
}

##########

sub crypturl
{
	# MM: I do not like to my username/password to appear anywhere
	# (especially in possible e-mails)
	my $url = shift @_;
	
	if (! ($url =~ s/(user(id)?)=$user/$1=XXXXXXXX/og))
	{
		my $user = uri_escape($user);
		if (! ($url =~ s/(user(id)?)=$user/$1=YYYYYYYY/og))
		{
			$url =~ s/(user(id)?)=[^\&]+/$1=ZZZZZZZZ/og;
		}
	}

	if (! ($url =~ s/pass=$pass/pass=XXXXXXXX/og))
	{
		my $pass = uri_escape($pass);
		if (! ($url =~ s/pass=$pass/pass=YYYYYYYY/og))
		{
			$url =~ s/pass=[^\&]+/pass=ZZZZZZZZ/;
		}
	}

	return $url;
}

##########
# Silent mode

sub silenton
{
        $silent = ($debug > 0) ? 0 : 1;
}

sub silentoff
{
        $silent = 0;
}

sub silentstatus
{
        return $silent;
}

sub silentset
{
        $silent = shift @_;
}

##########
# When in Track mode, there are some actions "prints" we do _not_ want
# to see happening ... those functions are here to let those happen only
# if not if Track mode

sub Track_log_message
{
	return if ($Track != -1);

	return log_message(@_);
}

sub Track_print
{
	return if ($Track != -1);	

	print @_;
}

##########
# Some print functions

sub dprint
{
	return if (! $debug);

	print STDERR "DEBUG: ", @_, "\n";
}

sub error_print
{
	print "ERROR: ", @_ ,"\n";
}

sub warn_print
{
	print "Warning: ", @_ ,"\n";
}

sub concat_strings
{
	my $text;

	while (my $tmp = shift @_)
	{
		$text .= $tmp;
	}

	return $text;
}

##########
# Exit functions

# Work with exit status
# (Used exit status where choosen checking /usr/include/sysexits.h) ie > 78
sub bidend_exit
{
	ok_exit() if (! $Exitstatus);

	my %item_info = @_;

	if ($item_info{'buyer'} =~ /^Private\sAuction$/)
	{
		log_message("This was a \"Private Auction\", it is not possible to know who the highest bidder was.\n", "unknown winner");
		ok_exit();
	}

	cleanup();
        # Did we win ?
        if ($item_info{'buyer'} =~ /^$user\s*\(/)
        {
		exit($Exitstatus_won);
	}
        else
        {
		exit($Exitstatus_lost);
	}
}

sub ok_exit
{
        cleanup();
	keypress_exit(0);
}

sub error_exit
{
	my $text;

	while ($text = shift @_)
	{
		print STDERR "$text";
	}

	cleanup();
	# In case of error we _always_ exit with the "normal" error status
	keypress_exit(1);
}

sub keypress_exit
{
	my $exit_status = shift @_;
	
	if ($keypress != 0)
	{
		print "\nPress \"ENTER\" to Finish\n";
		sysread(STDIN, $keypress, 1);
	}
	
	exit($exit_status);
}

sub cleanup
{
        $cjar->save() if ($debug > 2);
}

########################################
# Zealous

sub get_zealous_text
{
        my %item_info = @_;
        my $txt = "";

        $zealous_on = $zealous;
        $txt = checkitem(%item_info);
        $zealous_on = 0;

        return $txt;
}

########################################
# Locale (Part 2)

sub list_locale 
{
	my $tmp = "";

	$tmp .= "Currently recognized web sites (for bidding purpose) are:";
	foreach my $localeitem (@localelist)
	{
		$tmp .= " $localeitem";
	}
	$tmp .= "\n";

	return $tmp;
}

sub bidvalue_locale 
{
	my $bid = shift @_;

        $bid = sprintf("%.02f", $bid);

	if ($locale eq "com")
	{
		# Nothing (for now)
	}
	elsif ($locale eq "de")
	{ # Germany
		# Test for .de: not converting
		$bid =~ s/\./%2C/s;
	}
	
	return $bid;
}

sub bidvalue2_locale 
{
	my $bid = shift @_;

        $bid = sprintf("%.02f", $bid);

	if ($locale eq "com")
	{
		# Nothing (for now)
	}
	elsif ($locale eq "de")
	{ # Germany
		# Test for .de: not converting
		$bid =~ s/\./,/s;
	}
	
	return $bid;
}

sub bidpost1_locale 
{
	my ($item, $bid)  = @_;
	
	if ($locale =~ /^(com|de)$/)
	{
		return "MfcISAPICommand=MakeBid&fb=2&co_partnerid=&item=$item&maxbid=$bid";
	}
}

sub bidpost2_locale {
	my ($item, $key, $bid, $user, $pass)  = @_;

	if ($locale eq "com")
	{
# Was:		return "MfcISAPICommand=AcceptBid&item=$item&key=$key&maxbid=$bid&quant=1&userid=$user&pass=$pass";
		return "MfcISAPICommand=MakeBid&item=$item&key=$key&user=$user&pass=$pass&maxbid=$bid&quant=1&acceptbinbid=0&mode=1&userid=default&javascriptenabled=0&signin=y&BIN_button=PlaceBid";
	}
	elsif ($locale eq "de")
	{	
		return "BIN_button=Gebot+best%E4tigen&MfcISAPICommand=MakeBid&item=$item&maxbid=EUR%A0$bid&quant=1&userid=&key=$key&javascriptenabled=0&pass=$pass&user=$user&mode=1";
#		return "MfcISAPICommand=MakeBid&item=$item&maxbid=$bid&quant=1&userid=default&pass=$pass&key=$key&javascriptenabled=0&mode=1&user=$user&signin=y&BIN_button=Gebot+best%E4tigen";
	}
}

########################################
# Deamonize part ('Rest' option)

sub daemonize 
{
        chdir '/'
                or die "Can't chdir to /: $!";
        open STDIN, '/dev/null'
                or die "Can't read /dev/null: $!";
        open STDOUT, '>>/dev/null'
                or die "Can't write to /dev/null: $!";
        open STDERR, '>>/dev/null'
                or die "Can't write to /dev/null: $!";
        defined (my $pid = fork)
                or die "Can't fork: $!";

        exit if $pid;

        setsid() or die "Can't start a new session: $!";

        umask 0;
}

############################################################
# Now the documentation
__END__

=head1 DESCRIPTION

This program is designed to allow the user to I<bid> at the end of an eBay auction in order to get a better price when other bidders are pushing the price up incrementally.  It really doesn't help much when someone else has put in a high proxy bid.  What it does do is prevents other users from modifying their bids by bidding right at the end of the auction.

It works out when an auction will end by requesting the information about an item.  It looks at the date header sent by the eBay web server and also the end time of the auction.  The program compares the two times and sleeps until "gap" seconds before the end of the auction.

In this way it does not use the host systems time to, only eBay's time.

At the present time, S<eBayAgent> is still not made to work with C<Dutch Auctions> (checking S<user> in the bid winner list is not yet implemented) and have a restriction on C<Private Auctions> (since the highest bidder name is hidden, we can not know if S<user> has it, so your bid need to be over the minimal bid at all time).

=head1 OPTIONS

=over

=item B<-i>, B<--item>

Set the item number to bid on (in recent version of eBayAgent, the item number can simply be specified on the command line without being preceded by B<-i>. The default behavior if only an item number is given is to B<--check> that item).

=item B<-u>, B<--user>

Set the username to use when bidding.

=item B<-p>, B<--pass>

Set the password to use when bidding.

=item B<-l>, B<--locale>

eBay's locale web site to use to facilitate bidding for users of website different from the S<.com>. eBayAgent's default is to use S<.com> for all processings.

Note that currently, the script still I<check> the item using the S<.com> website, but it will I<bid> using the locale value.

Use S<--locale list> to obtain a list of currently supported locales.

=item B<-m>, B<--mail>

Turn on mailing of log messages to the user.

=item B<-M>, B<--Mailuser>

Specify the e-mail address to use when sending log message.

Overrides the eBay username.

Note that multiple e-mail addresses need to be separated them by a comma (,).

=item B<-S>, B<--Sendmail>

When sending mail, will use the Perl package S<Mail::Sendmail> to send e-mails instead of the UNIX command S<sendmail>.

If provided, will send e-mails via S<smtpserver>, better than S<localhost>.

Note that to use this package to send e-mails, B<Mailuser> must be a fully qualified e-mail (example: S<user@domain.name>).

For the B<rcfile>, if using S<localhost> as the S<smtpserver>, just type:
  Sendmail 1

The S<Mail::Sendmail> package is available at: S<http://search.cpan.org/search?dist=Mail-Sendmail>

=item B<-P>, B<--Proxy>

Allow users to use a proxy to access web sites. If no argument is provided, the software will use the S<http_proxy> environment variable (if available). Note that the software will not do any check on your HTTP proxy information, just provide it to S<LWP::UserAgent>, so you need to make sure that its syntax is ok with the module (ie, it should be of the form S<http://mycute.little.proxy.com:3128/>, where 3128 is the proxy port).

If you want the script to look up environment variables, set the value to 1 in the rcfile.

=item B<-f>, B<--fake_useragent>

This option allow you to change the default user agent used by eBayAgent. Simply it is the I<browser> identity sent by eBayAgent when requesting web pages.

If unsure which user agent header to use, using your favorite browser, simply go to: S<http://ebayagent.sourceforge.net/cgi-bin/check_UA>, where a copy of this simple CGI script is installed:

  #!/bin/sh
  
  echo "Content-type: text/plain"
  echo
  echo User agent: $HTTP_USER_AGENT

or you can monitor the traffic produced by your browser when accessing a web site, using either the S<tcpdump>:

  tcpdump -X -s 512 tcp and port http

and look for a I<User-Agent> text, or use S<ethereal> and check the first entry which I<protocol> is I<HTTP>, from your host to its destination, where I<info> starts with I<GET>, and look in the I<Hypertext Transfer Protocol> detail for the I<User-Agent> line.

=item B<-r>, B<--rcfile>

Set the file to read recurring data, like username, password and gap time.  The password option is especially nice because the username is not on the command line for all to see with the C<ps> listing.

For safety, make sure the rcfile is readable only by yourself.

Default search path for the S<.ebayrc> file is S<.> then S<$HOME>

Lines starting with C<#> are considered comments.

When using a on/off option, 0 is off and 1 is on. 

Recognized entries are I<user>, I<pass>, I<mail>, I<Mailuser>, I<gap>, I<dcheck>, I<acheck>, I<httpretry>, I<Hrdelay>, I<watch>, I<Watch>, I<smart>, I<Sendmail>, I<Adult>, I<Proxy>, I<fake_useragent>, I<ntponly>, I<locale>.

Example file (each line is optional and can be superseded by a command line option):
  user yourusername
  pass yourpassword
  gap  10
  mail 1
  Mailuser myemail@myfully.qualifiedaddress.net
  dcheck 20
  acheck 15
  Watch 30
  smart 1

This will work as user S<yourusername> with password S<yourpassword>, placing a bid S<10> seconds before the end of the bid, checking S<20> seconds before placing the bid for its validity, as well as S<15> seconds after the end of the bid to see who won, and send e-mails to S<myemail@myfully.qualifiedaddress.net>. During the entire bid process, the program will check every S<30> minutes to check that the bid is still valid and inform you of any price change. Also, if there were no bid on the item yet when you launched eBayAgent, since it has been started in S<smart> mode, it will place an initial bid at the minimal amount.

=item B<-b>, B<--bid>

Set the maximum bid amount. Please note that the bid amount specified is not in dollar but in the currency of the bid.

=item B<-s>, B<--smart>

If no bid has been placed on the item yet, the software will place a first bid at the minimum accepted bid amount. Note that B<smart> is a configuration file option and therefore using a command line argument with the value B<0> will turn it off.

=item B<-g>, B<--gap>

Set the gap to leave between bidding and the end of the auction in seconds.

=item B<-d>, B<--dcheck>

Check dcheck seconds before end of gap to see the value validity of our bid.

=item B<-a>, B<--acheck>

Check acheck seconds after end of bid to see who won the bid.

=item B<-N>, B<--Now>

Place a I<Buy It Now> bid. The function is to be used in conjunction with the B<--bid> option. The amount of the bid must be the I<Buy It Now> amount.

=item B<-h>, B<--httpretry>

When eBayAgent access the internet, it may encounter an empty page sent by eBay. This variable will set the number of HTTP request retries to perform (at B<--Hrdelay> interval) before considering that there may be a problem.

=item B<-H>, B<--Hrdelay>

This is to be used in conjunction with B<--httpretry> to specify the delay between retries (in seconds).

=item B<-n>, B<--ntponly>

Makes the program not trust eBay's clock after the first time the end time was asked. This is only safe to use when your system use a form of time synchronisation such as NTP (see C<http://www.ntp.org/> for more information).

=item B<-w>, B<--watch>

Check current price of the item every watch minutes to insure that the proposed bid amount is not under the bid amount.

=item B<-W>, B<--Watch>

Extension to the B<--watch> option; inform of all prices changes.

=item B<-T>, B<--Track>

Keep track of price changes for specified item only; does not authorize to place a bid. Will Track at specified interval to let you know of price changes for the specified item.

=item B<-z>, B<--zealous>

When in B<Track> or B<w/Watch> mode, notify of all changes (even changes in bidder/seller feedback information)

=item B<-B>, B<--Bidnow>

Place a bid at time of program call (will not perform a bid at the end of the auction) for specified item and amount (used with the B<--bid> option).

=item B<-E>, B<--Exitstatus>

Changes the returned exit status.
Returns 99 if the user won the bid (it is safer to check on this return value, because if an error occurred, the error return code will be used).
This option is useful if in case you want to bid to a second item of the same kind if you lost the first bid; you can use the S<status> (csh) to relaunch a second bid.
Example: C<eBayAgent.pl item1 -b price1 -E; if ($status != 99) eBayAgent.pl item2 -b price2>.

Note that this option can not work for C<Private Auctions> and will exit normaly without special exit status.

Thanks to B<Harald Milz>, a multiple item bid script called C<eBayAgent_Skript> is available and is distributed with the source of eBayAgent or should be placed in the doc directory (for RPMs installations)

=item B<-A>, B<--Adult>

Allow to bid for I<Adult Items>.

This means that you agree to abide by eBay's I<Mature Audiences Category Terms of Use> (as described when you try to access a I<Mature Audience> item). More information is available at: C<http://cgi3.ebay.com/aw-cgi/eBayISAPI.dll?AdultLoginShow&t=1>

In order for this option to work, you need to have verified yourself as an adult with eBay and have provided them with a valid credit card number. More information on the ID verification process is available at: C<http://pages.ebay.com/help/basics/f-verify.html>

=item B<-c>, B<--check>

Check the item information and exit (if only an item number is given, this is the default behavior).

=item B<-q>, B<--quiet>

Suppress the countdown display before the bid and before the check for the winner. This is useful if the output is redirected to a logfile.

=item B<-k>, B<--keypress>

Will wait for a user keypress before exiting script (usualy used by XeBayAgent)

=item B<-t>, B<--time>
	
Ask eBay for its time and exit

=item B<-R>, B<--Rest>

Start the bidding process in 'deamon' mode (program start and continue in the background). It is recommend to have mail set up with this method.

=item B<-D>, B<--Disclaimer>

print the program disclaimer and exit

=item B<-v>, B<--version>

print the version of the program and exit

=back

=head1 ADDITIONAL TOOLS

Additional tools (user provided) are available in the S<Tools> source distribution directory, or the S<Tools> rpm documentation directory (S</usr/share/doc> on RedHat 9.0 systems)

=head1 BUG REPORT & FEATURE REQUEST

Please send such reports or requests to C<martial@users.sourceforge.net>

=head1 NOTES

=head1 CHANGES

Check header of the Perl file for all applied changes and acknowledgments.

=head1 HOMEPAGE

Project homepage located at:
C<http://sourceforge.net/projects/ebayagent/>

=head1 AUTHORS

 Andrew Wansink <ajwans@users.sourceforge.net>
 Martial Michel <martial@users.sourceforge.net>

=head1 VERSION

$Id: eBayAgent-de.pl,v 1.1.2.1 2004/11/26 14:23:44 sturnus Exp $

=head1 COPYRIGHT

 (c) 2001-2004 Andrew Wansink & Martial Michel
 Other additions belonging to various authors (see CHANGES section).

=cut

## MM (2001/08/31): emacs mode (closest I came up in a quick try)
#;; eBayAgent.pl perl mode hook
#(defun ebayagent-mode ()
#  "CPerl mode to work with eBayAgent.pl identations style"
#  (interactive)
#  (cperl-mode)
#  (setq cperl-indent-level 8)
#  (setq cperl-brace-imaginary-offset 0)
#  (setq cperl-brace-offset -8)
#  (setq cperl-argdecl-indent 8)
#  (setq cperl-label-offset -8)
#  (setq cperl-continued-statement-offset 8)
#  (setq indent-tabs-mode nil)
#  (setq tab-width 8)
#  )
#(setq auto-mode-alist (cons '("eBayAgent.pl" . ebayagent-mode) auto-mode-alist))
